@@ -262,15 +262,23 @@ class BorutaShap:
             self.columns = self.X.columns.to_numpy()
             self.create_shadow_features()
 
+            # early stopping
             if self.X.shape[1] == 0:
                 break
 
             else:
 
-                try:
-                    self.model.fit(self.X_boruta, self.y, verbose=False)
-                except:
-                    self.model.fit(self.X_boruta, self.y)
+                # catboost needs extra parameters
+                if 'catboost' in str(type(self.model)).lower():
+                    self.model.fit(self.X_boruta, self.y, cat_features = self.X_categorical,  verbose=False)
+
+                else:
+
+                    try:
+                        self.model.fit(self.X_boruta, self.y, verbose=False)
+                    except:
+                        self.model.fit(self.X_boruta, self.y)
+
 
                 self.X_feature_import, self.Shadow_feature_import = self.feature_importance()
                 self.update_importance_history()
@@ -459,6 +467,9 @@ class BorutaShap:
         self.X_shadow = self.X.apply(np.random.permutation)
         self.X_shadow.columns = ['shadow_' + feature for feature in self.X.columns]
         self.X_boruta = pd.concat([self.X, self.X_shadow], axis = 1)
+        
+        col_types = self.X_boruta.dtypes
+        self.X_categorical = list(col_types[(col_types=='category' ) | (col_types=='object')].index)
 
 
     @staticmethod
@@ -500,7 +511,7 @@ class BorutaShap:
         if self.importance_measure == 'shap':
 
             self.explain()
-            vals = np.abs(self.shap_values).mean(0)
+            vals = self.shap_values
             vals = self.calculate_Zscore(vals)
 
             X_feature_import = vals[:len(self.X.columns)]
@@ -583,33 +594,61 @@ class BorutaShap:
         """
 
       
-        explainer = shap.TreeExplainer(self.model, approximate=True, feature_perturbation = "tree_path_dependent")
+        explainer = shap.TreeExplainer(self.model, feature_perturbation = "tree_path_dependent")
         
 
         if self.sample:
             
+
             if self.classification:
                 # for some reason shap returns values wraped in a list of length 1
                 
                 self.shap_values = np.array(explainer.shap_values(self.find_sample()))
-                if len(self.shap_values.shape) == 3: self.shap_values = self.shap_values.sum(axis=0)
+                if isinstance(self.shap_values, list):
+
+                    class_inds = range(len(self.shap_values))
+                    shap_imp = np.zeros(self.shap_values[0].shape[1])
+                    for i, ind in enumerate(class_inds):
+                        shap_imp += np.abs(self.shap_values[ind]).mean(0)
+                    self.shap_values /= len(self.shap_values)
+
+                elif len(self.shap_values.shape) == 3:
+                    self.shap_values = np.abs(self.shap_values).sum(axis=0)
+                    self.shap_values = self.shap_values.mean(0)
+                
+                else:
+                    self.shap_values = np.abs(self.shap_values).mean(0)
 
             else:
                 self.shap_values = explainer.shap_values(self.find_sample())
+                self.shap_values = np.abs(self.shap_values).mean(0)
             
         else:
             
             if self.classification:
                 # for some reason shap returns values wraped in a list of length 1
                 self.shap_values = np.array(explainer.shap_values(self.X_boruta))
-                if len(self.shap_values.shape) == 3: self.shap_values = self.shap_values.sum(axis=0)
+                if isinstance(self.shap_values, list):
+
+                    class_inds = range(len(self.shap_values))
+                    shap_imp = np.zeros(self.shap_values[0].shape[1])
+                    for i, ind in enumerate(class_inds):
+                        shap_imp += np.abs(self.shap_values[ind]).mean(0)
+                    self.shap_values /= len(self.shap_values)
+                
+                elif len(self.shap_values.shape) == 3:
+                    self.shap_values = np.abs(self.shap_values).sum(axis=0)
+                    self.shap_values = self.shap_values.mean(0)
+
+                else:
+                    self.shap_values = np.abs(self.shap_values).mean(0)
             
             else:
                 self.shap_values = explainer.shap_values(self.X_boruta)
+                self.shap_values = np.abs(self.shap_values).mean(0)
 
  
 
-
     @staticmethod
     def binomial_H0_test(array, n, p, alternative):
         """
@@ -741,11 +780,15 @@ class BorutaShap:
         self.accepted = self.accepted + newly_accepted.tolist()
 
 
-    def Subset(self):
+
+    def Subset(self, tentative=False):
         """
         Returns the subset of desired features
         """
-        return self.starting_X[self.accepted]
+        if tentative:
+            return self.starting_X[self.accepted + self.tentative.tolist()]
+        else:
+            return self.starting_X[self.accepted]
 
 
     @staticmethod
@@ -777,7 +820,7 @@ class BorutaShap:
 
     
     def plot(self, X_rotation=90, X_size=8, figsize=(12,8),
-            y_scale='log', which_features='all'):
+            y_scale='log', which_features='all', display=True):
 
         """
         creates a boxplot of the feature importances
@@ -798,6 +841,9 @@ class BorutaShap:
             Despite efforts if the number of columns is large the plot becomes cluttered so this parameter allows you to 
             select subsets of the features like the accepted, rejected or tentative features default is all.
 
+        Display: Boolean
+        controls if the output is displayed or not, set to false when running test scripts
+
         """
         # data from wide to long
         data = self.history_x.iloc[1:]
@@ -823,6 +869,10 @@ class BorutaShap:
                       X_size=X_size,
                       y_scale=y_scale,
                       figsize=figsize)
+        if display:
+            plt.show()
+        else:
+            plt.close()
         
 
     def box_plot(self, data, X_rotation, X_size, y_scale, figsize):
@@ -845,7 +895,6 @@ class BorutaShap:
         ax.set_title('Feature Importance')
         ax.set_ylabel('Z-Score')
         ax.set_xlabel('Features')
-        plt.show()
 
 
     def create_mapping_of_features_to_attribute(self, maps = []):
@@ -871,6 +920,7 @@ class BorutaShap:
         return dict(zip(list_one, list_two))
 
 
+
 def load_data(data_type='classification'):
 
     """
@@ -893,4 +943,4 @@ def load_data(data_type='classification'):
         raise ValueError("No data_type was specified, use either 'classification' or 'regression'")
   
 
-    return X, y
+    return X, y
\ No newline at end of file
